/*
 * Copyright (c) ROSA Laboratory 2011
 * License: GPLv3
 *
 * Authors:
 * Alexey Yermakov <alexey.yermakov@rosalab.ru>
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3,
 *   or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "rosa-starter.h"

#include <QtDBus/QtDBus>

#include <QPainter>
#include <QGraphicsWidget>
#include <QIcon>
#include <QObject>
#include <QTimer>
#include <QProcess>

#include <Plasma/IconWidget>

#include <KRun>

#include <klocalizedstring.h>

#include "config.h"

ROSA_Starter::ROSA_Starter(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args),
      m_preparedTimes(0)
{
  /*
  setBackgroundHints(NoBackground);
  setContentsMargins(0, 0, 0, 0);
  */
}

ROSA_Starter::~ROSA_Starter()
{
  if(hasFailedToLaunch()) {
    // Cleanup
  } else {
    // Save settings
  }
}

void ROSA_Starter::init()
{
  // Init

  // QTimer::singleShot(1000, this, SLOT(prepareLauncher()));
    connect(&m_prepareTimer, SIGNAL(timeout()), this, SLOT(prepareLauncher()));
    m_prepareTimer.start(1000);

  setHasConfigurationInterface(false);
  setAspectRatioMode(Plasma::Square);

  m_layout = new QGraphicsLinearLayout(Qt::Vertical);
  m_layout->setContentsMargins(0, 4, 0, 5);
  setLayout(m_layout);

  setMinimumWidth(10);
  setMinimumHeight(10);

  m_iconSize = 48;

  if(formFactor() == Plasma::Vertical)
    m_iconSize = topLevelWidget()->size().width();
  else if(formFactor() == Plasma::Horizontal)
    m_iconSize = topLevelWidget()->size().height();

  resize(m_iconSize, m_iconSize);

  int checkStatus = QProcess::execute(QString(EXTRA_PATH) + "checkos.sh");
  //int checkStatus = QProcess::execute(QString(EXTRA_PATH) + "checkos.sh", QStringList());

  m_icon = new Plasma::IconWidget();
  
  if(checkStatus == 0)
    m_icon->setIcon(QIcon(QString(PIXMAP_PATH) + "rosa-icon.png"));
  else
    m_icon->setIcon(QIcon(QString(PIXMAP_PATH) + "mandriva-icon.png"));

  m_layout->addItem(m_icon);

  connect(this, SIGNAL(geometryChanged()), this, SLOT(resetIconSize()));
  connect(m_icon, SIGNAL(clicked()), this, SLOT(buttonClicked()));
}

void ROSA_Starter::resetIconSize(void)
{
  //kDebug() << "resetIconSize CALLED!";
  m_icon->update(m_icon->boundingRect());
}

void ROSA_Starter::prepareLauncher(void)
{
  //kDebug() << "prepareLauncher CALLED!";
  QDBusConnection sessionBus = QDBusConnection::sessionBus();
  QDBusInterface *interface = new QDBusInterface("org.kde.ROSA_Launcher", "/MainApplication", "org.kde.KUniqueApplication", sessionBus, this);

  if(! interface->isValid()) // if SW is not launched
    {
        m_preparedTimes++;

        if(m_preparedTimes >= 20)
            m_prepareTimer.stop();
        
      /*
      QProcess *process = new QProcess(this);
      process->start("rosa-launcher -s");
      */

      KRun::runCommand("rosa-launcher -s", NULL);
    }
}

void ROSA_Starter::buttonClicked(void)
{
  /*
  kDebug() << "buttonClicked CALLED!";
  QDBusMessage message = QDBusMessage::createMethodCall("org.kde.screensaver", "/ScreenSaver", "org.freedesktop.ScreenSaver", "Lock");

  QDBusConnection::sessionBus().send(message);
  */

  QDBusConnection sessionBus = QDBusConnection::sessionBus();
  QDBusInterface *interface = new QDBusInterface("org.kde.ROSA_Launcher", "/MainApplication", "org.kde.KUniqueApplication", sessionBus, this);

  if(interface->isValid()) // if SW is launched
    {
      interface->call("newInstance");
    }
  else
    {
      QProcess *process = new QProcess(this);
      process->start("rosa-launcher");

      m_preparedTimes = 0;
      m_prepareTimer.start();
    }

  delete interface;
}

K_EXPORT_PLASMA_APPLET(rosastarter, ROSA_Starter)

#include "rosa-starter.moc"
